/* /**********************************************************************\
*                                Odatix                                *
************************************************************************
*
* Copyright (C) 2022 Jonathan Saussereau
*
* This file is part of Odatix.
* Odatix is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
* 
* Odatix is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with Odatix. If not, see <https://www.gnu.org/licenses/>.
*
*/


html, body {
    margin: 0;
    padding: 0;
    width: 100%;
    height: 100%;
    background: inherit;
    color: inherit;
}

.theme {
    background: var(--theme-background);
    color: var(--theme-text-color);
    font-family: var(--theme-font-family), Arial, sans-serif;
    text-shadow: var(--theme-text-shadow);
    font-size: var(--theme-font-size);
}

.subtle-text {
    color: var(--theme-subtle-text-color);
    font-size: var(--theme-subtle-text-font-size);
}

.navbar {
    background: var(--navbar-background);
    backdrop-filter: blur(10px);
    color: white;
    padding-right: 10px;
    display: flex;
    align-items: center;
    justify-content: space-between;
    margin: 0px; 
}

.navbar .nav-links {
    display: flex;
    margin-right: 20px;
}

.navbar .nav-links a {
    color: white;
    margin: 0 15px;
    text-decoration: none;
    font-size: 18px;
}

.navbar .nav-links a:hover {
    text-decoration: underline;
}

.navbar .title {
    position: relative;
    top: 0px;
    font-weight: bold;
    font-size: 24px;
    z-index: 2;
    color: white;
    text-decoration: none;
    overflow: hidden; 
    transition: transform 0.4s ease;
}

.link {
    height: 30px;
    align-items: center;
    color: #000;
    text-decoration: none;
    display: flex;
}

.mask {
    position: relative;
    padding: 0;
    height: 20px;
    overflow: hidden;
}

.link-container {
    transition: transform 0.4s ease;
}

.title {
    display: block;
    font-size: 20px;
    line-height: 20px;
    transition: transform 0.4s ease;
}

.link-title1 {
    transform-origin: right center;
}

.link-title2 {
    transform-origin: left center;
    transform: rotate(20deg);
}

.link:hover .link-container {
    transform: translateY(-20px);
}

.link:hover .link-title1 {
    transform: rotate(20deg);
}

.link:hover .link-title2 {
    transform: rotate(0);
}

.sidebar {
    position: relative;
    color: white;
    position: fixed;
    top: 0;
    height: 100%;
    padding-top: 10px;
    overflow-x: hidden;
    transition: left 0.25s;
}

.sidebar-content {
    margin-top: 50px;
    position: relative;
    color: white;
    position: fixed;
    top: 0;
    height: 100%;
    overflow-x: hidden;
    transition: left 0.25s;
}

.sidebar-content-holder {
    padding-bottom: 150px;
}

.sidebar-top{
    position: fixed;
    top: 0px;
    left: 0px;
    transition: left 0.25s;
}

.sidebar .title {
    display: flex;
    color: white;
    text-decoration: none;
    font-size: 24px;
    font-weight: bold;
    justify-content: center;
    align-items: center;
    text-align: center;
}

.sidebar h2 {
    margin-left: 20px;
    margin-top: 30px;
    margin-bottom: 15px;
}

.sidebar-button {
    transition: all .2s cubic-bezier(0.175, 0.885, 0, 1);
}
.sidebar-button:hover {
    box-shadow: 0px 30px 18px -8px rgba(0, 0, 0, 0.1);
    transform: scale(1.05, 1.05);
}

button {
    font-size: var(--button-font-size);
    padding: 10px 15px;
    margin-right: 5px;
    font-family: var(--theme-font-family), Arial;
    font-weight: 515;
    line-height: var(--button-line-height);
    cursor: pointer;
    border-radius: var(--button-border-radius);
    transition: var(--button-transition);
    box-shadow: var(--button-shadow-color);

    background: var(--default-button-background);
    border: var(--default-button-border);
    color: var(--default-button-text-color);
    --button-icon-color: var(--default-button-icon-color);
}
button:hover {
    background: var(--default-button-background-hover);
    border: var(--default-button-border-hover);
    color: var(--default-button-text-color-hover);
    --button-icon-color: var(--default-button-icon-color-hover);
}

input {
    font-family: var(--theme-font-family), Arial;
    font-size: var(--input-font-size);
    background: var(--input-background);
    border: var(--input-border);
    border-radius: var(--input-border-radius);
    color: var(--input-text-color);
    padding: 10px 5px;
    font-weight: 515;
    box-shadow: 0 1px 0 rgba(27, 31, 35, 0.04);
}

pre {
    font-family: var(--theme-font-family), Arial;
    font-size: var(--pre-font-size);  
    background: var(--pre-background);
    border: var(--pre-border);
    border-radius: var(--pre-border-radius);
    color: var(--pre-text-color);
    text-shadow: var(--pre-text-shadow);
    box-shadow: 0 1px 0 rgba(27, 31, 35, 0.04);
}

textarea {
    font-family: var(--theme-font-family), Arial;
    font-size: var(--textarea-font-size);  
    background: var(--textarea-background);
    border: var(--textarea-border);
    border-radius: var(--textarea-border-radius);
    color: var(--textarea-text-color);
    padding: 10px 5px;
    font-weight: 515;
    box-shadow: 0 1px 0 rgba(27, 31, 35, 0.04);
}

.title-dropdown{
    display: flex;
    vertical-align: middle;
    align-items: center;
    margin-left: 75px;
    margin-right: 75px;
}

.sidebar .title-dropdown{
    display: flex;
    vertical-align: middle;
    align-items: center;
    margin-left: 20px;
    margin-right: 20px;
}

.dropdown-label{
    margin: var(--dropdown-label-margin);
}

.dash-dropdown{
    font-size: var(--dropdown-font-size); 
    margin-bottom: 5px;
    width: 250px;
    font-weight: 600;
    cursor: pointer;
    transition: background-color 0.2s, border-color 0.2s, box-shadow 0.2s; 
    box-shadow: 0 1px 0 rgba(27, 31, 35, 0.04); 
    appearance: none;   
}

.dash-dropdown, .Select, .Select-control, .Select-input input {
    border: none !important;
    border-radius: var(--main-border-radius);
    background-color: #00000000 !important;
}

.Select-value {
    background-color: var(--dropdown-background);
    border: var(--dropdown-border);
    border-radius: var(--dropdown-border-radius);
    color: var(--theme-text-color);
}

.Select-value-label {
    color: var(--theme-text-color) !important;
}

.Select-menu-outer {
    background-color: var(--dropdown-menu-background);
    backdrop-filter: var(--dropdown-menu-backdrop-filter);
    border: var(--dropdown-border);
    border-radius: var(--dropdown-menu-border-radius);
    color: var(--theme-text-color);
}

.Select-value:hover {
    background-color: var(--dropdown-background-hover);
    border-color: var(--main-border-color-hover);
}

.error {
    background-color: var(--pre-background);
    color: #ef553b;
    border: var(--pre-border);
    border-radius: var(--pre-border-radius); 
    text-shadow: var(--pre-text-shadow);
    font-weight: 600;
    margin: 10px;
    margin-left: 20px;
    margin-right: 20px;
    padding: 10px;
    box-shadow: 0 1px 0 rgba(27, 31, 35, 0.04); 
}

.error.note {
    color: #228BE6;
}

.error.warning {
   color: #FD7E14;
}

.error pre {
    font-family: monospace, monospace;
    font-weight: 500;
}

.error details {
    background-color: #fff;
    color: #24292e;
    border: 1px solid #d1d5da;
    border-radius: 6px; 
    font-weight: 600;
    margin: 10px;
    margin-left: 0px;
    margin-right: 0px;
    padding: 10px;
    box-shadow: 0 1px 0 rgba(27, 31, 35, 0.04); 
}

.content {
    transition: margin-left 0.25s
}

.toggle-container {
    margin-left: 15px;
}

.tile.center {
    margin: 25px;
    max-width: fit-content; 
    margin-inline: auto;
    font-weight: bold;
    justify-content: center;
    align-items: center;
    text-align: center;
}
.tile.center h1 {
    font-size: 100px; 
    margin: 10px;
}
.tile.center h2 {
    font-size: 40px; 
    margin: 20px;
}

.home-container {
    display: flex;
    flex-direction: column;
    align-items: center;
    justify-content: center;
    text-align: center;
}

.card {
    background-color: var(--card-background);
    color: var(--card-text-color);
    min-width: 0;
    max-width: 100%;
    box-shadow: var(--card-box-shadow);
    border: var(--card-border);
    border-radius: var(--card-border-radius);
    backdrop-filter: var(--card-backdrop-filter);
    overflow: visible;
    display: flex;
    flex-direction: column;
    justify-content: space-between;
    text-decoration: none;
    transition: all .2s cubic-bezier(0.175, 0.885, 0, 1);
}
.card.add {
    background: var(--add-card-background);
    border: var(--add-card-border);
    color: var(--card-text-color);
}
.card.configs {
    width: 256px;
    overflow: visible !important;
    /* height: 200px; */
}
.card.configs.add {
    width: 277px;
}
.card.configs.wide {
    width: 705px;
}
.card.configs.add.wide {
    width: calc(705px + 21px);
}

/* @media screen and (max-width: 600px) {

    .card.configs.wide {
    justify-self: center;
        display: block;
        width: 67%;
        max-width: 67%;
        margin-right: 20px! important;
    }
    .card.configs.add.wide {
    justify-self: center;
        display: block;
        width: 70%;
        max-width: 70%;
        margin-right: 20px! important;
    }
} */

.card-img {
    margin: auto;
    object-fit: cover;
}

.card-title  {
    text-align: center;
    font-size: 1.25em;
    font-weight: bold;
}

.card-description {
    text-align: center;
    margin: 25px;
}

.card-matrix {
    display: grid;
    grid-template-columns: repeat(auto-fit, 350px);
    gap: 10px;
    justify-content: center; 
}
.card-matrix.config {
    grid-template-columns: repeat(auto-fit, 750px);
    gap: 0px;
}
.card-matrix.configs {
    grid-template-columns: repeat(auto-fit, 300px);
    gap: 0px;
}
.card-matrix.configs.wide {
    grid-template-columns: repeat(auto-fit, 750px);
    gap: 0px;
}

.home.card {
    width: 450px;
    height: 320px;
}

.card.hover:hover {
    cursor: pointer;
    box-shadow: var(--card-hover-box-shadow);
    transform: var(--card-hover-transform);
}

.overlay-odatix {
    position: fixed;
    top: 0; left: 0; right: 0; bottom: 0;
    background: var(--overlay-background); 
    backdrop-filter: var(--overlay-backdrop-filter);
    box-shadow: var(--overlay-box-shadow);
    transition: var(--overlay-transition);
    visibility: hidden;
    opacity: 0;
    z-index: 9999;
    display: flex;
    align-items: center;
    justify-content: center;
}
.overlay-odatix.visible {
    visibility: visible;
    opacity: 1;
}
.popup-odatix {
    background: var(--popup-background);
    border: var(--popup-border);
    border-radius: var(--popup-border-radius);
    box-shadow: var(--popup-box-shadow);
    backdrop-filter: var(--popup-backdrop-filter);
    width: 30vw;
    min-width: 320px;
    max-width: 90vw;
    padding: 24px;
    position: relative;
    text-align: center;
}
.popup-odatix h3 {
    margin-top: 0;
    color: #333;
}
.popup-odatix .close {
    position: absolute;
    top: 16px;
    right: 24px;
    font-size: 30px;
    font-weight: bold;
    text-decoration: none;
    color: #333;
    background: none;
    border: none;
    cursor: pointer;
}
.popup-odatix .close:hover {
    color: #FA5252;
}
@media screen and (max-width: 700px){
    .popup-odatix{
        width: 70vw;
        min-width: unset;
    }
}

.tile {
    display: block;
    width: 80%;
    max-width: 500px;
    margin-bottom: 16px;
    padding: 24px 32px;
    background-color: var(--card-background);
    color: var(--card-text-color);
    box-shadow: var(--card-box-shadow);
    border: var(--card-border);
    border-radius: var(--card-border-radius);
    break-inside: avoid;
    clip-path: path('M10% 50% C14% 22%, 32% 14%, 46% 28% C60% 18%, 78% 22%, 84% 44% C92% 58%, 82% 74%, 64% 84% C50% 94%, 32% 90%, 20% 76% C8% 66%, 6% 58%, 10% 50% Z');
    transition: 
        /* width 0.4s cubic-bezier(0.4, 0.2, 0.2, 1), */
        height 0.4s cubic-bezier(0.4, 0.2, 0.2, 1),
        transform 0.4s cubic-bezier(0.4, 0.2, 0.2, 1);
}

.tile.hover:hover {
    cursor: pointer;
    box-shadow: var(--card-hover-box-shadow);
    transform: var(--card-hover-transform);
}

.tile.config {
    justify-self: center;
    max-width: 660px;
    width: 660px;
}

.tile.title {
    justify-self: center;
    max-width: 1410px;
    width: 1410px;
    height: fit-content;
}
.tile.title.add {
    background: var(--add-card-background);
    border: var(--add-card-border);
    color: var(--card-text-color);
}

.tile.title.hover {
    transition: var(--card-hover-transition);
}

.tiles-container {
    column-width: 450px;
    column-gap: 16px;
    width: 100%;
    max-width: 1000px;
    margin: auto;
}

.tiles-container.config {
    column-width: 600px;
    column-gap: 10px;
    width: 100%;
    max-width: 1488px;
    margin: auto;
}

.title-input {
    margin-top: -10px;
    font-size: 24px;
    font-weight: bold;
    background-color: var(--title-input-background);
    border: var(--title-input-border);
    box-shadow: none;
}

.title-input.domain {
    width: 500px;
}

.title-input:hover {
    background-color: var(--title-input-background-hover);
    border: var(--title-input-border-hover);
}
.title-input.preview:hover {
    border: 1px solid #ffffff00;
}

.title-input:focus {
    background-color: var(--title-input-background-focus);
    border: var(--title-input-border-focus);
    outline: none;
}
.title-input.preview:focus {
    background-color: transparent;
    border: 1px solid #ffffff00;
}

.inline-flex-buttons {
    display: inline-flex;
    margin-left: 16px;
    vertical-align: middle;
    margin-right: 0;
    justify-content: flex-end;
    width: 100px;
}
@media screen and (max-width: 2205px) {
    .tile.title {
        width: 1110px;
        max-width: 1110px;
    }
    .tile.config {
        max-width: 510px;
        width: 510px;
    }
    .tiles-container.config {
        column-width: 450px;
        max-width: 1187px;
    }
    .card-matrix.config {
        grid-template-columns: repeat(auto-fit, 600px);
        gap: 0px;
    }
    .card.configs.wide {
        width: 554px;
        max-width: 554px;
    }
    .card.configs.add.wide {
        width: calc(554px + 21px);
        max-width: calc(554px + 21px);
    }
    .card-matrix.configs.wide {
        grid-template-columns: repeat(auto-fit, 600px);
        gap: 0px;
    }
}
@media screen and (max-width: 1765px) {
    .tile.title {
        width: 810px;
        max-width: 810px;
    }
    .tile.config {
        max-width: 810px;
        width: 810px;
    }
    .tiles-container.config {
        column-width: 600px;
        max-width: 890px;
    }
}
@media screen and (max-width: 1324px) {
    .tile.title {
        width: 510px;
        max-width: 510px;
    }
    .tile.config {
        max-width: 510px;
        width: 510px;
    }
    .tiles-container.config {
        column-width: 600px;
        max-width: 590px;
    }
    .title-input {
        width: 275px;
    }
    .title-tile-flex {
        flex-direction: column !important;
        align-items: flex-start !important;
        gap: 10px;
    }
    .title-tile-flex > div {
        width: 100%;
    }
    .inline-flex-buttons {
        justify-content: flex-start !important;
        /* width: 100% !important; */
        margin-top: 10px !important;
        margin-left: 0 !important;
        margin-right: 0 !important;
    }
    .inline-flex-buttons.param-domain-title {
        margin-top: 0 !important;
    }
}
@media screen and (max-width: 882px) {
    .tile.config {
        margin-left: -20px;
        width: 390px;
        max-width: 390px;
    }
    .tiles-container.config {
        column-width: 600px;
        max-width: 420px;
    }
    .tile.title {
        margin-left: 10px;
        width: 390px;
        max-width: 390px;
    }
    .card.configs {
        padding: 32px;
        width: calc(390px + 44px);
        max-width: calc(390px + 44px);
    }
    .card.configs.wide {
        padding: 32px;
        width: calc(390px + 44px);
        max-width: calc(390px + 44px);
    }
    .card-matrix.configs {
        grid-template-columns: none;
        gap: 0px;
    }
    .card-matrix.configs.wide {
        grid-template-columns: none;
        gap: 0px;
    }
}

.animated-section {
    transition: max-height 0.4s cubic-bezier(0.4, 0.2, 0.2, 1), opacity 0.3s;
    max-height: none; 
    opacity: 1;
}

.animated-section.hide {
    max-height: 0;
    opacity: 0;
    pointer-events: none;
    overflow: hidden;
}

.animated-section.no-margin {
    margin-top: 0 !important;
    margin-bottom: 0 !important;
}

h3 {
    margin-top: 0px;
}

.status{
    color: rgb(134, 142, 150);
    margin-left: 16px;
}
.status.error {
    color: var(--theme-caution-color);
}
.status.warning {
    color: var(--theme-warning-color);
}
.status.valid {
    color: var(--theme-success-color);
}

.text-highlight {
    border-radius: var(--highlight-border-radius);
    font-weight: var(--highlight-font-weight);
}
.text-highlight.primary {
    background: var(--primary-highlight-background);
    border: var(--primary-highlight-border);
    color: var(--primary-highlight-text-color);
    text-shadow: var(--primary-highlight-text-shadow);
}
.text-highlight.secondary {
    background: var(--secondary-highlight-background);
    border: var(--secondary-highlight-border);
    color: var(--secondary-highlight-text-color);
    text-shadow: var(--secondary-highlight-text-shadow);
}

.tooltip {
    position: relative;
}

.tooltip .icon {
    cursor: help;
    fill: var(--tooltip-icon-color);
    filter: var(--tooltip-icon-filter);
}

.tooltip::after {
    font-size: var(--tooltip-font-size);
    font-weight: normal;
    line-height: var(--tooltip-line-height);
    text-shadow: var(--tooltip-text-shadow);
    box-shadow: var(--tooltip-box-shadow);
    border-radius: var(--tooltip-border-radius);
    width: var(--tooltip-normal-width);
    padding: var(--tooltip-padding);
    content: attr(data-tooltip);
    position: absolute;
    bottom: 120%;
    left: 50%;
    transform: translateX(-50%);
    opacity: 0;
    pointer-events: none;
    transition: opacity 0.25s;
    background: var(--default-tooltip-background);
    color: var(--default-tooltip-text-color);
    border: var(--default-tooltip-border);
    z-index: 10;
}
.tooltip.delay::after {
    transition-delay: 0s;
}

.tooltip.auto::after {
    width: auto;
}
.tooltip.small::after {
    width: var(--tooltip-small-width);
}
.tooltip.large::after {
    width: var(--tooltip-large-width);
}
.tooltip.bottom::after {
    top: 120%;
    bottom: auto;
}
.tooltip.primary::after {
    background: var(--primary-tooltip-background);
    color: var(--primary-tooltip-text-color);
    border: var(--primary-tooltip-border);
}
.tooltip.secondary::after {
    background: var(--secondary-tooltip-background);
    color: var(--secondary-tooltip-text-color);
    border: var(--secondary-tooltip-border);
}
.tooltip.warning::after {
    background: var(--warning-tooltip-background);
    color: var(--warning-tooltip-text-color);
    border: var(--warning-tooltip-border);
}
.tooltip.caution::after {
    background: var(--caution-tooltip-background);
    color: var(--caution-tooltip-text-color);
    border: var(--caution-tooltip-border);
}
.tooltip.error-status::after {
    background: var(--error-tooltip-background);
    color: var(--error-tooltip-text-color);
    border: var(--error-tooltip-border);
}

.tooltip:hover::after {
    opacity: 1;
}
.tooltip.delay:hover::after {
    transition-delay: 0.5s;
}

.icon-button {
    width: 39px;
    height: 39px;
}
.icon-label-button {
    min-width: 300px;
    height: 39px;
}

.color-button.caution {
    background: var(--caution-button-background);
    border: var(--caution-button-border);
    color: var(--caution-button-text-color);
    --button-icon-color: var(--caution-button-icon-color);
    box-shadow: var(--caution-button-box-shadow);
}
.color-button.caution:hover {
    background: var(--caution-button-background-hover);
    border: var(--caution-button-border-hover);
    color: var(--caution-button-text-color-hover);
    --button-icon-color: var(--caution-button-icon-color-hover);
    box-shadow: var(--caution-button-box-shadow-hover);
}

.color-button.warning {
    background: var(--warning-button-background);
    border: var(--warning-button-border);
    color: var(--warning-button-text-color);
    --button-icon-color: var(--warning-button-icon-color);
    box-shadow: var(--warning-button-box-shadow);
    animation: var(--warning-button-animation);
}
.color-button.warning:hover {
    background: var(--warning-button-background-hover);
    border: var(--warning-button-border-hover);
    color: var(--warning-button-text-color-hover);
    --button-icon-color: var(--warning-button-icon-color-hover);
    box-shadow: var(--warning-button-box-shadow-hover);
}

.color-button.success {
    background: var(--success-button-background);
    border: var(--success-button-border);
    color: var(--success-button-text-color);
    --button-icon-color: var(--success-button-icon-color);
    box-shadow: var(--success-button-box-shadow);
    animation: var(--success-button-animation);
}
.color-button.success:hover {
    background: var(--success-button-background-hover);
    border: var(--success-button-border-hover);
    color: var(--success-button-text-color-hover);
    --button-icon-color: var(--success-button-icon-color-hover);
    box-shadow: var(--success-button-box-shadow-hover);
}

.color-button.primary {
    background: var(--primary-button-background);
    border: var(--primary-button-border);
    color: var(--primary-button-text-color);
    --button-icon-color: var(--primary-button-icon-color);
    box-shadow: var(--primary-button-box-shadow);
}
.color-button.primary:hover {
    background: var(--primary-button-background-hover);
    border: var(--primary-button-border-hover);
    color: var(--primary-button-text-color-hover);
    --button-icon-color: var(--primary-button-icon-color-hover);
    box-shadow: var(--primary-button-box-shadow-hover);
}
.color-button.secondary {
    background: var(--secondary-button-background);
    border: var(--secondary-button-border);
    color: var(--secondary-button-text-color);
    --button-icon-color: var(--secondary-button-icon-color);
    box-shadow: var(--secondary-button-box-shadow);
}
.color-button.secondary:hover {
    background: var(--secondary-button-background-hover);
    border: var(--secondary-button-border-hover);
    color: var(--secondary-button-text-color-hover);
    --button-icon-color: var(--secondary-button-icon-color-hover);
    box-shadow: var(--secondary-button-box-shadow-hover);
}

.color-button.disabled {
    cursor: default;
    background: var(--disabled-button-background);
    border: var(--disabled-button-border);
    color: var(--disabled-button-text-color);
    --button-icon-color: var(--disabled-button-icon-color);
    box-shadow: var(--disabled-button-box-shadow);
}
.color-button.disabled:active {
    pointer-events: none;
}
.color-button.error-status {
    cursor: not-allowed !important;
    background: var(--error-button-background);
    border: var(--error-button-border);
    color: var(--error-button-text-color);
    --button-icon-color: var(--error-button-icon-color);
    box-shadow: var(--error-button-box-shadow);
}
.color-button.error-status:active {
    pointer-events: none;
}

.color-button.invisible {
    display: none;
}

.icon.black {
    fill: #24292e;
}
.icon.red {
    fill: #FA5252;
}
.icon.blue {
    fill: #228BE6;
}
.icon.orange {
    fill: #FD7E14;
}
.icon.green {
    fill: #40C057;
}
.icon.grey {
    fill: #868E964D;
}
.icon.disabled {
    fill: #868E964D;
}
.icon.primary {
    fill: var(--theme-primary-color);
}
.icon.secondary {
    fill: var(--theme-secondary-color);
}
.icon.tertiary {
    fill: var(--theme-tertiary-color);
}
.icon.accent {
    fill: var(--theme-accent-color);
}
.icon.action {
    fill: var(--theme-action-color);
}
.icon.warning {
    fill: var(--theme-warning-color);
}
.icon.caution {
    fill: var(--theme-caution-color);
}
.icon.success {
    fill: var(--theme-success-color);
}

.icon.back-button {
    fill: var(--back-button-color);
    filter: var(--back-button-filter);
    transition: transform 0.2s cubic-bezier(0.4, 0.2, 0.2, 1);
}
.icon.icon.back-button:hover {
    transform: scale(1.25);
    filter: var(--back-button-filter-hover);
}

.icon-button .icon {
    transition: var(--button-transition);
    fill: var(--button-icon-color);
    cursor: grab;
}

.icon-button.error-status .icon {
    cursor: not-allowed;
}

.icon-button.disabled .icon {
    cursor: default;
}

.icon.rotate {
    transition: transform 0.3s ease;
}
.icon.rotated {
    transform: rotate(180deg);
}
