/* /**********************************************************************\
*                                Odatix                                *
************************************************************************
*
* Copyright (C) 2022 Jonathan Saussereau
*
* This file is part of Odatix.
* Odatix is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
* 
* Odatix is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with Odatix. If not, see <https://www.gnu.org/licenses/>.
*
*/

@font-face {
    font-family: 'Nunito';
    src: url('fonts/Nunito-SemiBold.ttf') format('truetype');
    font-weight: normal;
    font-style: normal;
}

@font-face {
    font-family: 'W95';
    src: local('W95'),
        url('fonts/w95/w95font.woff2') format('woff2'),
        url('fonts/w95/w95font.woff') format('woff');
    font-weight: normal;
    font-style: normal;
    font-display: swap;
}

.theme {
    /* theme */
    --theme-font-family: Nunito;
    --theme-large-font-size: 16px;
    --theme-default-font-size: 14px;
    --theme-small-font-size: 11px;
    --theme-background-color: #f5f7fa;
    --theme-element-background-color: #fff; /* mainly cards and tiles background */
    --theme-dimmed-element-background-color: #fff0;
    --theme-action-color: #d1d5da;
    --theme-primary-color: #228BE6; /* accent color for tooltips and switches */
    --theme-secondary-color: #228BE6; /* accent color for important buttons */
    --theme-tertiary-color: #f5f7fa; /* background color for fields  */
    --theme-disabled-color: #798ba64f;
    --theme-disabled-error-color: #fa5252ab;
    --theme-disabled-background-color: var(--theme-tertiary-color);
    --theme-disabled-error-background-color: #FA525229;
    --theme-success-color: #40C057;
    --theme-warning-color: #FF8825;
    --theme-caution-color: #FA5252;
    --theme-error-color: var(--theme-caution-color);
    --theme-shadow-color: rgba(29, 37, 79, 0.1);
    --theme-text-color: #24292e;
    --theme-subtle-text-color: #cbcfd4;
    --theme-subtle-text-font-size: var(--theme-small-font-size);
    --theme-disabled-text-color: var(--theme-disabled-color);
    --theme-contrast-background-color: #24292e;
    --theme-contrast-text-color: #fff;
    --theme-contrast-text-shadow: 0 1px 2px #000000a6, 0 0 1px #fff;
    --theme-text-shadow: none;
    --theme-background: var(--theme-background-color);
    --theme-border-color: #dde3ec;
    --theme-border-style: 1px solid;
    --theme-large-border-color: var(--theme-border-color);
    --theme-large-border-style: 2px solid;
    --theme-border: var(--theme-border-style) var(--theme-border-color);
    --theme-border-color-hover: #24292e;
    --theme-large-border-radius: 24px;
    --theme-normal-border-radius: 24px;
    --theme-small-border-radius: 15px;
    --theme-overlay-background-color: #1d254f3d;
    
    /* navbar */
    --navbar-background: var(--theme-contrast-background-color);
    --navbar-text-color: var(--theme-background-color);

    /* cards and tiles */
    --card-background: var(--theme-element-background-color);
    --card-text-color: var(--theme-text-color);
    --card-border: initial;
    --card-box-shadow: var(--theme-shadow-color) 0px 4px 24px 0px;
    --card-border-hover: none;
    --card-border-radius: var(--theme-large-border-radius);
    --card-backdrop-filter: initial;
    --card-hover-box-shadow: 0px 30px 18px -8px rgba(0, 0, 0, 0.1);
    --card-hover-transition: all 0.2s cubic-bezier(0.175, 0.885, 0, 1);
    --card-hover-transform: scale(1.05, 1.05);
    --add-card-background: var(--theme-dimmed-element-background-color);
    --add-card-border: 1px dashed #bbb;
    --add-card-text-color: var(--card-text-color);

    /* buttons */
    --button-font-size: var(--theme-default-font-size);
    --button-shadow-color: var(--theme-shadow-color);
    --button-box-shadow: var(--button-shadow-color) 0px 0px 8px 0px;
    --button-transition: 0.3s;
    --button-border-radius: var(--theme-normal-border-radius);
    --button-line-height: 1.2em;
    --default-button-background: #fff0;
    --default-button-border: var(--theme-large-border-style) var(--theme-border-color);
    --default-button-icon-color: var(--theme-text-color);
    --default-button-text-color: var(--theme-text-color);
    --default-button-box-shadow: var(--button-box-shadow);
    --default-button-background-hover: #fff0;
    --default-button-border-hover: var(--theme-large-border-style) var(--theme-text-color);
    --default-button-icon-color-hover: var(--theme-text-color);
    --default-button-text-color-hover: var(--theme-text-color);
    --default-button-box-shadow-hover: var(--button-box-shadow);
    --primary-button-background: var(--theme-primary-color);
    --primary-button-border: var(--theme-large-border-style) #fff0;
    --primary-button-icon-color: var(--theme-contrast-text-color);
    --primary-button-text-color: var(--theme-contrast-text-color);
    --primary-button-box-shadow: var(--button-box-shadow);
    --primary-button-background-hover: #fff0;
    --primary-button-border-hover: var(--theme-large-border-style) var(--theme-primary-color);
    --primary-button-icon-color-hover: var(--theme-primary-color);
    --primary-button-text-color-hover: var(--theme-primary-color);
    --primary-button-box-shadow-hover: var(--button-box-shadow);
    --secondary-button-background: var(--theme-secondary-color);
    --secondary-button-border: var(--theme-large-border-style) #fff0;
    --secondary-button-icon-color: var(--theme-contrast-text-color);
    --secondary-button-text-color: var(--theme-contrast-text-color);
    --secondary-button-box-shadow: var(--button-box-shadow);
    --secondary-button-background-hover: #fff0;
    --secondary-button-border-hover: var(--theme-large-border-style) var(--theme-secondary-color);
    --secondary-button-icon-color-hover: var(--theme-secondary-color);
    --secondary-button-text-color-hover: var(--theme-secondary-color);
    --secondary-button-box-shadow-hover: var(--button-box-shadow);
    --success-button-background: var(--theme-success-color);
    --success-button-border: var(--theme-large-border-style) #fff0;
    --success-button-icon-color: var(--theme-contrast-text-color);
    --success-button-text-color: var(--theme-contrast-text-color);
    --success-button-box-shadow: var(--button-box-shadow);
    --success-button-background-hover: #fff0;
    --success-button-border-hover: var(--theme-large-border-style) var(--theme-success-color);
    --success-button-icon-color-hover: var(--theme-success-color);
    --success-button-text-color-hover: var(--theme-success-color);
    --success-button-box-shadow-hover: var(--button-box-shadow);
    --warning-button-background: var(--theme-warning-color);
    --warning-button-border: var(--theme-large-border-style) #fff0;
    --warning-button-icon-color: var(--theme-contrast-text-color);
    --warning-button-text-color: var(--theme-contrast-text-color);
    --warning-button-box-shadow: var(--theme-warning-color) 0px 0px 10px;
    --warning-button-animation: pulse 2s ease infinite both -0.5s;
    --warning-button-background-hover: #fff0;
    --warning-button-border-hover: var(--theme-large-border-style) var(--theme-warning-color);
    --warning-button-icon-color-hover: var(--theme-warning-color);
    --warning-button-text-color-hover: var(--theme-warning-color);
    --warning-button-box-shadow-hover: var(--theme-warning-color) 0px 0px 10px;
    --caution-button-background: var(--theme-caution-color);
    --caution-button-border: var(--theme-large-border-style) var(--theme-caution-color);
    --caution-button-icon-color: var(--theme-contrast-text-color);
    --caution-button-text-color: var(--theme-contrast-text-color);
    --caution-button-box-shadow: var(--button-box-shadow);
    --caution-button-background-hover: #fff0;
    --caution-button-border-hover: var(--theme-large-border-style) var(--theme-caution-color);
    --caution-button-icon-color-hover: var(--theme-caution-color);
    --caution-button-text-color-hover: var(--theme-caution-color);
    --caution-button-box-shadow-hover: var(--button-box-shadow);
    --error-button-background: var(--theme-disabled-error-background-color);
    --error-button-border: var(--theme-large-border-style) #fff0;
    --error-button-icon-color: var(--theme-disabled-error-color);
    --error-button-text-color: var(--theme-disabled-error-color);
    --error-button-box-shadow: none;
    --disabled-button-background: var(--theme-disabled-background-color);
    --disabled-button-border: var(--theme-large-border-style) #fff0;
    --disabled-button-icon-color: var(--theme-disabled-text-color);
    --disabled-button-text-color: var(--theme-disabled-text-color);
    --disabled-button-box-shadow: var(--button-box-shadow);
    --back-button-color: var(--theme-disabled-color);
    --back-button-filter: drop-shadow(0px 0px 2px var(--theme-shadow-color)) drop-shadow(0px 0px 5px var(--theme-shadow-color));
    --back-button-filter-hover: drop-shadow(0px 0px 2px var(--theme-shadow-color)) drop-shadow(0px 0px 5px var(--theme-shadow-color));

    /* dropdowns */
    --dropdown-font-size: var(--theme-default-font-size);
    --dropdown-background: var(--theme-background-color);
    --dropdown-background-hover: var(--theme-action-color);
    --dropdown-border: var(--theme-border);
    --dropdown-border-radius: var(--theme-normal-border-radius);
    --dropdown-label-margin: 0px;
    --dropdown-menu-background: var(--theme-element-background-color);
    --dropdown-menu-backdrop-filter: blur(24px);
    --dropdown-menu-border-radius: var(--theme-small-border-radius);
    --dropdown-menu-hover: var(--theme-action-color);

    /* inputs */
    --input-font-size: var(--theme-default-font-size);
    --input-background: var(--theme-tertiary-color);
    --input-text-color: var(--theme-text-color);
    --input-border: var(--theme-border);
    --input-border-radius: var(--theme-small-border-radius);
    --title-input-background: transparent;
    --title-input-text-color: var(--theme-text-color);
    --title-input-border: var(--theme-border-style) #ffffff00;
    --title-input-border-radius: var(--theme-small-border-radius);
    --title-input-background-hover: transparent;
    --title-input-text-color-hover: var(--theme-text-color);
    --title-input-border-hover: var(--theme-border);
    --title-input-border-radius-hover: var(--theme-small-border-radius);
    --title-input-background-focus: var(--theme-tertiary-color);
    --title-input-text-color-focus: var(--theme-text-color);
    --title-input-border-focus: var(--theme-border);
    --title-input-border-radius-focus: var(--theme-small-border-radius);

    /* switch */
    --switch-off-background: var(--theme-tertiary-color);
    --switch-on-background: #228BE614;
    --switch-off-handle-background: var(--theme-disabled-color);
    --switch-on-handle-background: var(--theme-primary-color);
    --switch-off-border-style: var(--theme-border-style);
    --switch-on-border-style: var(--theme-border-style);

    /* textareas */
    --textarea-font-size: var(--theme-small-font-size);  
    --textarea-background: var(--theme-tertiary-color);
    --textarea-text-color: var(--theme-text-color);
    --textarea-border: var(--theme-border);
    --textarea-border-radius: var(--theme-small-border-radius);

    /* pre */
    --pre-font-size: var(--theme-small-font-size);
    --pre-background: var(--theme-tertiary-color);
    --pre-text-color: var(--theme-text-color);
    --pre-border: var(--theme-border);
    --pre-border-radius: var(--theme-small-border-radius);
    --pre-text-shadow: none;

    /* tooltip */
    --tooltip-font-size: var(--theme-large-font-size);
    --tooltip-icon-color: var(--theme-primary-color);
    --tooltip-icon-filter: drop-shadow(0 0 2px var(--theme-shadow-color));
    --tooltip-text-shadow: var(--theme-contrast-text-shadow);
    --tooltip-border-radius: var(--theme-small-border-radius);
    --tooltip-box-shadow: 0 4px 12px var(--theme-shadow-color);
    --tooltip-large-width: 450px;
    --tooltip-normal-width: 300px;
    --tooltip-small-width: 150px;
    --tooltip-padding: 10px 15px;
    --tooltip-line-height: 1.2em;
    --default-tooltip-background: var(--theme-contrast-background-color);
    --default-tooltip-text-color: var(--theme-contrast-text-color);
    --default-tooltip-border: none;
    --primary-tooltip-background: var(--theme-primary-color);
    --primary-tooltip-text-color: var(--theme-contrast-text-color);
    --primary-tooltip-border: none;
    --secondary-tooltip-background: var(--theme-secondary-color);
    --secondary-tooltip-text-color: var(--theme-contrast-text-color);
    --secondary-tooltip-border: none;
    --caution-tooltip-background: var(--theme-caution-color);
    --caution-tooltip-text-color: var(--theme-contrast-text-color);
    --caution-tooltip-border: none;
    --warning-tooltip-background: var(--theme-warning-color);
    --warning-tooltip-text-color: var(--theme-contrast-text-color);
    --warning-tooltip-border: none;
    --error-tooltip-background: var(--theme-error-color);
    --error-tooltip-text-color: var(--theme-contrast-text-color);
    --error-tooltip-border: none;

    /* popups */
    --popup-background: var(--theme-element-background-color);
    --popup-border: var(--theme-border);
    --popup-border-radius: var(--theme-large-border-radius);
    --popup-box-shadow: 0 2px 12px #0002;
    --popup-backdrop-filter: initial;
    --overlay-background: var(--theme-overlay-background-color);
    --overlay-backdrop-filter: blur(4px);
    --overlay-box-shadow: #1d254f0f 0px 32px 48px 0px;
    --overlay-transition: opacity 500ms;

    /* highlights */
    --highlight-font-weight: 700;
    --highlight-border-radius: 6px;
    --primary-highlight-color: #FFE65E;
    --primary-highlight-background: var(--primary-highlight-color);
    --primary-highlight-border: 2px solid var(--primary-highlight-color);
    --primary-highlight-text-color: var(--theme-text-color);
    --primary-highlight-text-shadow: var(--theme-text-shadow);
    --secondary-highlight-color: var(--theme-primary-color);
    --secondary-highlight-background: none;
    --secondary-highlight-border: none;
    --secondary-highlight-text-color: var(--secondary-highlight-color);
    --secondary-highlight-text-shadow: var(--theme-text-shadow);
}

.theme.odatix {
    /* default theme */
}

.theme.odatix_dark {
    /* theme */
    --theme-background-color: #24292e;
    --theme-element-background-color: #33383c;
    --theme-action-color: #0078d4;
    --theme-primary-color: #0078d4;
    --theme-secondary-color: #0078d4;
    --theme-tertiary-color: #43474a;
    --theme-disabled-color: #616b73;
    --theme-text-color: #ffffff;
    --theme-text-shadow: 0 1px 2px #000, 0 0 1px #fff;
    --theme-border-color: #24292e;
    --theme-border: 0.5px solid var(--theme-border-color);
    --theme-overlay-background-color: #33383c3d;
    --theme-contrast-background-color: #181b20;

    /* cards and tiles */
    --card-box-shadow: rgba(11, 11, 15, 0.27) 0px 4px 24px 0px;

    /* highlights */
    --primary-highlight-color: var(--theme-primary-color);
    --primary-highlight-text-color: var(--theme-contrast-text-color);
    --secondary-highlight-color: var(--theme-warning-color);
    --secondary-highlight-text-shadow: var(--theme-contrast-text-shadow);

    /* buttons */
    --back-button-color: var(--theme-disabled-color);
}

.theme.odatix_darker {
    /* theme */
    --theme-background-color: #17181a;
    --theme-element-background-color: #242628;
    --theme-action-color: #0078d4;
    --theme-primary-color: #005799;
    --theme-secondary-color: #005799;
    --theme-tertiary-color: #2f3133;
    --theme-disabled-color: #515760;
    --theme-warning-color: #D27120;
    --theme-caution-color: #CA3939;
    --theme-text-color: #ffffff;
    --theme-text-shadow: 0 1px 2px #000, 0 0 1px #fff;
    --theme-border-color: #3a3d40;
    --theme-border: none;
    --theme-overlay-background-color: #33383c3d;
    --theme-contrast-background-color: #0b0b0b;

    /* cards and tiles */
    --card-box-shadow: rgba(11, 11, 15, 0.27) 0px 4px 24px 0px;

    /* buttons */
    --button-background: var(--theme-tertiary-color);
    --back-button-color: var(--theme-disabled-color);

    /* inputs */
    --title-input-border: var(--theme-border-style) #ffffff00;
    --title-input-border-hover: var(--theme-border-style) var(--theme-border-color);
    --title-input-border-focus: var(--theme-border-style) #ffffff00;


    /* tooltips */
    --tooltip-text-shadow: none;

    /* highlights */
    --primary-highlight-color: var(--theme-primary-color);
    --primary-highlight-text-color: var(--theme-contrast-text-color);
    --secondary-highlight-color: var(--theme-warning-color);
    --secondary-highlight-text-shadow: var(--theme-contrast-text-shadow);
}

.theme.catpuccin {
	--rosewater: #f5e0dc;
	--flamingo: #f2cdcd;
	--pink: #f5c2e7;
	--mauve: #cba6f7;
	--red: #f38ba8;
	--maroon: #eba0ac;
	--peach: #fab387;
	--yellow: #f9e2af;
	--green: #a6e3a1;
	--teal: #94e2d5;
	--sky: #89dceb;
	--sapphire: #74c7ec;
	--blue: #89b4fa;
	--lavender: #b4befe;
	--text: #cdd6f4;
	--subtext1: #bac2de;
	--subtext0: #a6adc8;
	--overlay2: #9399b2;
	--overlay1: #7f849c;
	--overlay0: #6c7086;
	--surface2: #585b70;
	--surface1: #45475a;
	--surface0: #313244;
	--base: #1e1e2e;
	--mantle: #181825;
	--crust: #11111b;
	--inverted-text: var(--crust);
	--selection-color: var(--mauve);

    /* theme */
    --theme-background-color: var(--base);
    --theme-element-background-color: var(--surface0);
    --theme-action-color: var(--mauve);
    --theme-primary-color: var(--mauve);
    --theme-secondary-color: var(--blue);
    --theme-tertiary-color: var(--surface1);
    --theme-disabled-color: var(--surface0);
    --theme-success-color: var(--green);
    --theme-warning-color: var(--peach);
    --theme-caution-color: var(--red);
    --theme-text-color: var(--text);
    --theme-text-shadow: 0 1px 2px #000, 0 0 1px #fff;
    --theme-border-color: var(--surface2);
    --theme-overlay-background-color: #3132443d;
    --theme-contrast-background-color: var(--crust);
    --theme-contrast-text-color: var(--inverted-text);
    --theme-large-border-radius: 12px;
    --theme-normal-border-radius: 24px;
    --theme-small-border-radius: 15px;

    /* cards and tiles */
    --card-box-shadow: none;

    /* buttons */
    --hover-percentage: 95%;
    --default-button-icon-color: var(--theme-contrast-text-color);
    --default-button-text-color: var(--theme-contrast-text-color);
    --default-button-background: linear-gradient(120deg,var(--overlay1),var(--overlay2));
    --default-button-border-hover: var(--theme-large-border-style) #fff0;
    --default-button-icon-color-hover: var(--theme-contrast-text-color);
    --default-button-text-color-hover: var(--theme-contrast-text-color);
    --default-button-background-hover: linear-gradient(120deg,var(--overlay1) var(--hover-percentage),var(--overlay2));
    --default-button-border: var(--theme-large-border-style) #fff0;
    --primary-button-icon-color: var(--theme-contrast-text-color);
    --primary-button-text-color: var(--theme-contrast-text-color);
    --primary-button-background: linear-gradient(120deg,var(--blue),var(--sky));
    --primary-button-border-hover: var(--theme-large-border-style) #fff0;
    --primary-button-icon-color-hover: var(--theme-contrast-text-color);
    --primary-button-text-color-hover: var(--theme-contrast-text-color);
    --primary-button-background-hover: linear-gradient(120deg,var(--blue) var(--hover-percentage),var(--sky));
    --primary-button-border: var(--theme-large-border-style) #fff0;
    --secondary-button-icon-color: var(--theme-contrast-text-color);
    --secondary-button-text-color: var(--theme-contrast-text-color);
    --secondary-button-background: linear-gradient(120deg,var(--blue),var(--sky));
    --secondary-button-border: var(--theme-large-border-style) #fff0;
    --secondary-button-icon-color-hover: var(--theme-contrast-text-color);
    --secondary-button-text-color-hover: var(--theme-contrast-text-color);
    --secondary-button-background-hover: linear-gradient(120deg,var(--blue) var(--hover-percentage),var(--sky));
    --secondary-button-border-hover: var(--theme-large-border-style) #fff0;
    --warning-button-icon-color: var(--theme-contrast-text-color);
    --warning-button-text-color: var(--theme-contrast-text-color);
    --warning-button-background: linear-gradient(120deg,var(--peach),var(--yellow));
    --warning-button-border: var(--theme-large-border-style) #fff0;
    --warning-button-icon-color-hover: var(--theme-contrast-text-color);
    --warning-button-text-color-hover: var(--theme-contrast-text-color);
    --warning-button-background-hover: linear-gradient(120deg,var(--peach) var(--hover-percentage),var(--yellow));
    --warning-button-border-hover: var(--theme-large-border-style) #fff0;
    --caution-button-icon-color: var(--theme-contrast-text-color);
    --caution-button-text-color: var(--theme-contrast-text-color);
    --caution-button-background: linear-gradient(120deg,var(--red) var(--hover-percentage),var(--maroon));
    --caution-button-border: var(--theme-large-border-style) #fff0;
    --caution-button-icon-color-hover: var(--theme-contrast-text-color);
    --caution-button-text-color-hover: var(--theme-contrast-text-color);
    --caution-button-background-hover: linear-gradient(120deg,var(--red) var(--hover-percentage),var(--maroon));
    --caution-button-border-hover: var(--theme-large-border-style) #fff0;
    --back-button-color: var(--overlay0);

    /* highlights */
    --primary-highlight-color: var(--theme-primary-color);
    --primary-highlight-text-color: var(--theme-contrast-text-color);
    --secondary-highlight-color: var(--theme-secondary-color);
    --secondary-highlight-text-shadow: var(--theme-contrast-text-shadow);
}

.theme.dracula {
    /* theme */
    --theme-background-color: #272935;
    --theme-element-background-color: #34384c;
    --theme-action-color: #ff79c6;
    --theme-primary-color: #ff79c6;
    --theme-secondary-color: #404f6d;
    --theme-tertiary-color: #40455e;
    --theme-warning-color: var(--theme-primary-color);
    --theme-caution-color: #d2504d;
    --theme-disabled-color: #34384cac;
    --theme-disabled-background-color: #6272a44f;
    --theme-text-color: #f8f8f2;
    --theme-text-shadow: 0 1px 2px #000, 0 0 1px #fff;
    --theme-border-color: #4d516a;
    --theme-overlay-background-color: #282a363d;
    --theme-contrast-background-color: #23232c;

    /* cards and tiles */
    --card-box-shadow: rgba(11, 11, 15, 0.27) 0px 4px 24px 0px;

    /* highlights */
    --primary-highlight-text-color: var(--theme-contrast-text-color);
    --primary-highlight-color: #E5D776;
    --secondary-highlight-color: #5ab8ff;

    /* buttons */
    --secondary-button-border-hover: var(--theme-large-border-style) var(--theme-contrast-text-color);
    --secondary-button-icon-color-hover: var(--theme-contrast-text-color);
    --secondary-button-text-color-hover: var(--theme-contrast-text-color);
    --back-button-color: #6272a4aa;

    /* highlights */
    --primary-highlight-color: var(--theme-primary-color);
    --primary-highlight-text-color: var(--theme-contrast-text-color);
    --secondary-highlight-text-shadow: var(--theme-contrast-text-shadow);
}

.theme.code_dark {
    /* theme */
    --theme-background-color: #181818;
    --theme-element-background-color: #252526;
    --theme-action-color: #0078d4;
    --theme-primary-color: #0078d4;
    --theme-secondary-color: #0078d4;
    --theme-tertiary-color: #313131;
    --theme-disabled-color: #515760;
    --theme-success-color: #73c991;
    --theme-warning-color: #cca700;
    --theme-caution-color: #f88070;
    --theme-text-color: #ffffff;
    --theme-text-shadow: 0 1px 2px #000, 0 0 1px #fff;
    --theme-border: none;
    --theme-border-style: 0.5px solid;
    --theme-large-border-style: 0.5px solid;
    --theme-border-color: #d1d5da;
    --theme-small-border-radius: 3px;
    --theme-normal-border-radius: 3px;
    --theme-large-border-radius: 3px;
    --theme-overlay-background-color: #3131313d;

    /* navbar */
    --navbar-background: #1f1f1f;

    /* cards and tiles */
    --card-box-shadow: initial;

    /* buttons */
    --button-border: 0.5px solid var(--theme-disabled-color);
    --default-button-icon-color: var(--theme-text-color);
    --default-button-text-color: var(--theme-text-color);
    --default-button-background: var(--theme-tertiary-color);
    --default-button-border: var(--button-border);
    --default-button-border-hover: 0.5px solid var(--theme-text-color);
    --back-button-color: var(--theme-disabled-color);

    /* inputs */
    --title-input-border: var(--theme-border-style) #ffffff00;
    --title-input-border-hover: var(--theme-border-style) var(--theme-disabled-color);
    --title-input-border-focus: var(--theme-border-style) #ffffff00;

    /* tooltips */
    --default-tooltip-background: var(--theme-tertiary-color);

    /* highlights */
    --primary-highlight-text-color: var(--theme-background-color);
    --secondary-highlight-text-shadow: var(--theme-contrast-text-shadow);
}

.theme.rainbow {
    /* theme */
    --theme-element-background-color: #FFFFFF5E;
    --theme-dimmed-element-background-color: #ffffff0d;
    --theme-tertiary-color: #FFFFFF61;
    --theme-disabled-color: #65718259;
    --theme-border-color: #bac2ce59;
    --background-rainbow-intensity: 0.5;
    --theme-background: linear-gradient(
        90deg,
        rgba(255, 0, 0, var(--background-rainbow-intensity)) 0%,
        rgba(255, 154, 0, var(--background-rainbow-intensity)) 10%,
        rgba(208, 222, 33, var(--background-rainbow-intensity)) 20%,
        rgba(79, 220, 74, var(--background-rainbow-intensity)) 30%,
        rgba(63, 218, 216, var(--background-rainbow-intensity)) 40%,
        rgba(47, 201, 226, var(--background-rainbow-intensity)) 50%,
        rgba(28, 127, 238, var(--background-rainbow-intensity)) 60%,
        rgba(95, 21, 242, var(--background-rainbow-intensity)) 70%,
        rgba(186, 12, 248, var(--background-rainbow-intensity)) 80%,
        rgba(251, 7, 217, var(--background-rainbow-intensity)) 90%,
        rgba(255, 0, 0, var(--background-rainbow-intensity)) 100%
    );
    --theme-large-border-radius: 50px;
    --theme-small-border-radius: 20px;

    /* buttons */
    --default-button-icon-color: var(--theme-text-color);
    --default-button-text-color: var(--theme-text-color);
    --default-button-background: var(--theme-tertiary-color);
    --back-button-color: #fdfdfdc4;
}

.theme.galaxy {
    /* theme */
    --theme-background-color: #0b0716;
    --theme-element-background-color: rgba(255, 255, 255, 0.1);
    --theme-dimmed-element-background-color: rgba(255,255,255,0.02);
    --theme-action-color: #7A5CFF;
    --theme-primary-color: #7A5CFF;
    --theme-secondary-color: #7A5CFF;
    --theme-tertiary-color: #dbd7ea24;
    --theme-disabled-color: #ddccf652;
    --theme-success-color: #32d5a1;
    --theme-warning-color: #ffb86b;
    --theme-caution-color: #ff5c5c;
    --theme-text-color: #f8f3ff;
    --theme-text-shadow: 0 1px 3px rgba(0,0,0,0.7), 0 0 8px rgba(255,110,199,0.08);
    --theme-border-color: rgba(255,255,255,0.06);
    --theme-overlay-background-color: rgba(10,6,24,0.75);
    --theme-contrast-background-color: #0b0720;

    /* borders & radii */
    --theme-border: 1px solid rgba(255,255,255,0.04);
    --theme-large-border-radius: 20px;
    --theme-normal-border-radius: 12px;
    --theme-small-border-radius: 8px;

    /* background */
    --theme-background: linear-gradient(135deg, #12002b 0%, #2b0057 50%, #531f62 100%) no-repeat fixed;
    /* --theme-background: url("nightsky.svg") no-repeat fixed center / cover; */

    /* buttons */
    --default-button-icon-color: var(--theme-contrast-text-color);
    --default-button-text-color: var(--theme-contrast-text-color);
    --default-button-background: var(--theme-tertiary-color);
    --back-button-color: var(--theme-disabled-color);

    /* popups */
    --popup-backdrop-filter: blur(24px);
    --popup-background: rgba(181, 153, 191, 0.44);
    --overlay-backdrop-filter: none;
    --overlay-background: rgba(10, 6, 24, 0.40);

    /* highlights */
    --primary-highlight-color: var(--theme-primary-color);
    --primary-highlight-text-color: var(--theme-contrast-text-color);
    --secondary-highlight-color: var(--theme-warning-color);
}

.theme.midnight {
    /* theme */
    --theme-background-color: #121226;
    --theme-element-background-color: #1b1b2f;
    --theme-dimmed-element-background-color: #1b1b2fbb;
    --theme-action-color: #0077f6;
    --theme-primary-color: #0077f6;
    --theme-secondary-color: #0077f6;
    --theme-tertiary-color: #252544;
    --theme-disabled-color: #555577;
    --theme-text-color: #e0e0ff;
    --theme-text-shadow: 0 1px 2px #000, 0 0 1px #fff;
    --theme-background: linear-gradient(135deg, #0f0f1a 0%, #1b1b2f 100%);
    --theme-border: none;
    --theme-border-color: #555577;
    --theme-overlay-background-color: #121226aa;

    /* navbar */
    --navbar-background: #0b0b15;

    /* cards and tiles */
    --card-box-shadow: rgba(11, 11, 15, 0.5) 0px 4px 24px 0px;

    /* buttons */
    --button-background: var(--theme-tertiary-color);
    --back-button-color: var(--theme-disabled-color);
}

.theme.win95 {
    /* win95 palette */
    --win95-bg: #008080;
    --win95-face: #c0c0c0;
    --win95-blue: #0000a8;
    --win95-highlight: #ffffff;
    --win95-shadow: #808080;

    /* theme */
    --theme-font-family: W95;
    --theme-background-color: var(--win95-bg);
    --theme-element-background-color: var(--win95-face);
    --theme-dimmed-element-background-color: var(--win95-face);
    --theme-action-color: var(--win95-face);
    --theme-primary-color: var(--win95-blue);
    --theme-secondary-color: var(--win95-blue);
    --theme-tertiary-color: #d4d0c8;
    --theme-disabled-color: #808080;
    --theme-text-color: #000000;
    --theme-text-shadow: none;
    --theme-border-color: #000000;
    --theme-overlay-background-color: #0000003d;
    --theme-contrast-background-color: #000000;
    --theme-large-border-radius: 0px;
    --theme-normal-border-radius: 0px;
    --theme-small-border-radius: 0px;
    image-rendering: pixelated;

    /* cards and tiles */
    --card-box-shadow: none;
    --card-hover-transform: none;
    --card-hover-box-shadow: none;

    /* buttons */
    --button-border-radius: 0px;
    --default-button-background: var(--win95-face);
    --primary-button-background: var(--win95-face);
    --warning-button-animation: none;
    --secondary-button-background: var(--win95-face);
    --secondary-button-icon-color: var(--theme-secondary-color);
    --secondary-button-text-color: var(--theme-text-color);
    --warning-button-background: var(--win95-face);
    --warning-button-icon-color: var(--theme-warning-color);
    --warning-button-text-color: var(--theme-warning-color);
    --warning-button-box-shadow: none;
    --warning-button-box-shadow-hover: none;
    --caution-button-background: var(--win95-face);
    --caution-button-icon-color: var(--theme-caution-color);
    --caution-button-text-color: var(--theme-caution-color);

    /* dropdowns */
    --dropdown-background: var(--win95-face);
    --dropdown-menu-background: var(--win95-face);

    /* popups */
    --overlay-backdrop-filter: none;
    --overlay-transition: none;

    /* tooltips */
    --default-tooltip-background: var(--win95-face);
    --default-tooltip-text-color: var(--theme-text-color);
    --tooltip-text-shadow: none;

    /* highlights */
    --primary-highlight-color: var(--theme-primary-color);
    --primary-highlight-text-color: var(--theme-contrast-text-color);
    --secondary-highlight-color: var(--theme-warning-color);
    --secondary-highlight-text-shadow: var(--theme-contrast-text-shadow);
}
.theme.win95 .card,
.theme.win95 .tile,
.theme.win95 .popup-odatix,
.theme.win95 .Select-value,
.theme.win95 .Select-arrow-zone,
.theme.win95 .Select-menu,
.theme.win95 .tooltip:after,
.theme.win95 button {
    border-top: 2px solid var(--win95-highlight) !important;
    border-left: 2px solid var(--win95-highlight) !important;
    border-bottom: 2px solid var(--win95-shadow) !important;
    border-right: 2px solid var(--win95-shadow) !important;
}
.theme.win95 pre,
.theme.win95 input,
.theme.win95 textarea,
.theme.win95 button:active {
    border-top: 2px solid var(--win95-shadow) !important;
    border-left: 2px solid var(--win95-shadow) !important;
    border-bottom: 2px solid var(--win95-highlight) !important;
    border-right: 2px solid var(--win95-highlight) !important;
    transform: translateY(1px);
}
.theme.win95 .Select-arrow {
    color: #000;
}
.theme.win95 .Select-input input {
    border: none !important;
}

.theme.hangover {

}

.theme.legacy {
    /* theme */
    --theme-font-family: Arial;
    --theme-large-border-radius: 10px;
    --theme-normal-border-radius: 10px;
    --theme-small-border-radius: 10px;
    --theme-disabled-color: #ddd;

    /* cards and tiles */
    --card-border: 1px solid #ddd;
    --card-box-shadow: 0px 13px 10px -7px rgba(0, 0, 0, 0.1);
    --card-border-hover: 1px solid #ddd;
}
