/**********************************************************************\
*                                Odatix                                *
************************************************************************
*
* Copyright (C) 2022 Jonathan Saussereau
*
* This file is part of Odatix.
* Odatix is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
* 
* Odatix is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with Odatix. If not, see <https://www.gnu.org/licenses/>.
*
*/

.main-container {
  --main-bg-color: #ffffff;
  --main-text-color: #ffffff;
  --main-border-color: #24292e;
  --main-border-color-hover: #24292e;
  --main-border-radius: 10px;
  --home-bg-color: #f6f8fa;
  --card-bg-color: #ffffff;
  --card-text-color: #24292e;
  --card-border: 1px solid #ddd;
  --card-border-hover: 1px solid #ddd;
  --card-border-radius: 10px;
  --button-bg-color: #545c64;
  --button-bg-color-hover: #2f567d;
  --dropdown-bg-color: #636b71;
  --dropdown-bg-color-hover: #2f567d;
  --dropdown-menu-bg-color: #636b71;
  --sidebar-button-bg-filter: none;
}

.main-container.code_dark {
  --main-bg-color: #252526;
  --main-text-color: #ffffff;
  --main-border-color: #3c3c3c;
  --main-border-color-hover: #0078d4;
  --main-border-radius: 3px;
  --home-bg-color: #252526;
  --card-bg-color: #313131;
  --card-text-color: #ffffff;
  --card-border: 1px solid #00000000;
  --card-border-hover: 1px solid #0078d4;
  --card-border-radius: 2px;
  --button-bg-color: #0078d4;
  --button-bg-color-hover: #026ec1;
  --dropdown-bg-color: #313131;
  --dropdown-bg-color-hover: #313131;
  --dropdown-menu-bg-color: #313131;
}

.main-container.odatix_dark {
  --main-bg-color: #252526;
  --main-text-color: #ffffff;
  --main-border-color: #24292e;
  --main-border-color-hover: #24292e;
  --home-bg-color: #24292e;
  --card-bg-color: #393c40;
  --card-text-color: #ffffff;
  --card-border: 1px solid #00000000;
  --card-border-hover: 1px solid #00000000;
  --button-bg-color: #24292e;
  --button-bg-color-hover: #026ec1;
  --dropdown-bg-color: #24292e;
  --dropdown-bg-color-hover: #026ec1;
  --dropdown-menu-bg-color: #24292e;
}

.main-container.odatix_darker {
  --button-bg-color: #1b1e22;
  --dropdown-bg-color: #1b1e22;
  --dropdown-menu-bg-color: #1b1e22;
  --home-bg-color: #181b20;
  --card-bg-color: #24292e;
  --card-text-color: #ffffff;
  --card-border: 1px solid #00000000;
  --card-border-hover: 1px solid #00000000;
}

.main-container.odatix_light {
  --main-bg-color: #ffffff;
  --main-text-color: #24292e;
  --main-border-color: #d1d5da;
  --main-border-color-hover: #c6cbd1;
  --button-bg-color: #f6f8fae1;
  --button-bg-color-hover: #d1d5dae1;
  --dropdown-bg-color: #f6f8fae1;
  --dropdown-bg-color-hover: #d1d5dae1;
  --dropdown-menu-bg-color: #f6f8fae1;
  --sidebar-button-bg-filter: invert(0%) brightness(0%);
  --main-backdrop-filter: blur(8px);
}

.main-container.candy {
  --home-bg-color: linear-gradient(249deg,rgba(238, 174, 202, 1) 0%, rgba(148, 187, 233, 1) 100%);
  --main-border-color: #00000000;
  --main-border-color-hover: #00000000;
  --main-border-radius: 20px;
  --card-bg-color: #DFA2FC;
  --card-border: 1px solid #00000000;
  --card-border-hover: 1px solid #00000000;
  --card-border-radius: 50px;
  --button-bg-color: #DFA2FC;
  --button-bg-color-hover: #FCA2E7;
  --dropdown-bg-color: #DFA2FC;
  --dropdown-bg-color-hover: #FCA2E7;
  --dropdown-menu-bg-color: #af58e9;
}

.main-container.gradient {
  --home-bg-color: linear-gradient(45deg,rgba(238, 174, 202, 1) 0%, rgba(148, 187, 233, 1) 100%);
  --main-border-color: #00000000;
  --main-border-color-hover: #00000000;
  --main-text-color: #24292e;
  --card-bg-color: #ffffffc5;
  --button-bg-color: #bbc4cc5c;
  --button-bg-color-hover: #bbc4ccac;
  --dropdown-menu-bg-color: #bbc4cc5c;
  --dropdown-bg-color-hover: #bbc4ccac;
  --dropdown-bg-color: #bbc4cc5c;
  --main-backdrop-filter: blur(5px);
  --sidebar-button-bg-filter: invert(0%) brightness(0%);
}